@@ -29,6 +29,7 @@ from .protocol.const import (
 from .protocol.exceptions import CorruptStateError
 from .protocol.opentherm import MSG_ID, MSG_NAME, MSG_TYPE, OPENTHERM_MESSAGES, VALUE
 from .protocol.ramses import CODE_ONLY_FROM_CTL, RAMSES_DEVICES
+from .protocol.transport import PacketProtocolPort
 
 from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
@@ -52,6 +53,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
     _10A0,
     _10E0,
@@ -64,6 +66,8 @@ from .protocol import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
@@ -96,7 +100,7 @@ DEFAULT_BDR_ID = "13:000730"
 DEFAULT_EXT_ID = "17:000730"
 DEFAULT_THM_ID = "03:000730"
 
-DEV_MODE = __dev_mode__ and False
+DEV_MODE = __dev_mode__
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -175,6 +179,20 @@ class DeviceBase(Entity):
             return NotImplemented
         return self.id < other.id
 
+    def _start_discovery(self) -> None:
+
+        delay = randint(10, 20)
+
+        self._gwy._add_task(  # 10E0/1FC9, 3220 pkts
+            self._discover, discover_flag=Discover.SCHEMA, delay=0, period=3600 * 24
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.PARAMS, delay=delay, period=3600 * 6
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.STATUS, delay=delay + 1, period=60
+        )
+
     @discover_decorator
     def _discover(self, discover_flag=Discover.ALL) -> None:
         # sometimes, battery-powered devices will respond to an RQ (e.g. bind mode)
@@ -284,11 +302,11 @@ class DeviceInfo:  # 10E0
 
     def _discover(self, discover_flag=Discover.ALL) -> None:
         if discover_flag & Discover.SCHEMA:
-            try:
-                if RP in RAMSES_DEVICES[self.type][_10E0]:  # DEX (convert to e.g. BDR)
-                    self._make_cmd(_10E0, retries=3)
-            except KeyError:
-                pass
+            if not self._msgs.get(_10E0) and (
+                self.type not in RAMSES_DEVICES
+                or RP in RAMSES_DEVICES[self.type].get(_10E0, {})
+            ):  # DEX (convert to e.g. BDR)
+                self._make_cmd(_10E0, retries=3)
 
     @property
     def device_info(self) -> Optional[dict]:  # 10E0
@@ -397,13 +415,18 @@ class Actuator:  # 3EF0, 3EF1
             # NOTE: No need to send periodic RQ/3EF1s to an OTB, use RQ/3220/11s
             self._make_cmd(_3EF1)  # NOTE: No RPs to RQ/3EF0
 
-    def _handle_msg(self, msg) -> None:
+    def _handle_msg(self, msg) -> None:  # NOTE: active
         super()._handle_msg(msg)
 
         if isinstance(self, OtbGateway):
             return
 
-        if msg.code == _3EF0 and msg.verb == I_ and not self._faked:
+        if (
+            msg.code == _3EF0
+            and msg.verb == I_
+            and not self._faked
+            and not self._gwy.config.disable_sending
+        ):
             self._make_cmd(_3EF1, priority=Priority.LOW, retries=1)
 
     @property
@@ -695,13 +718,14 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
                 Command.get_relay_demand(self.id), priority=Priority.LOW, retries=1
             )
 
-    def _handle_msg(self, msg) -> None:
+    def _handle_msg(self, msg) -> None:  # NOTE: active
         if msg.src.id == self.id:
             super()._handle_msg(msg)
             return
 
         if (
-            not self._faked
+            self._gwy.config.disable_sending
+            or not self._faked
             or self._domain_id is None
             or self._domain_id
             not in (v for k, v in msg.payload.items() if k in ("domain_id", "zone_idx"))
@@ -915,7 +939,7 @@ class Controller(Device):  # CTL (01):
         self._domain_id = "FF"
         self._evo = None
 
-        self._make_tcs_controller()
+        self._make_tcs_controller(**kwargs)
 
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
@@ -1116,10 +1140,11 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.temperature}"
 
-    def _handle_msg(self, msg) -> None:
+    def _handle_msg(self, msg) -> None:  # NOTE: active
         super()._handle_msg(msg)
 
-        if msg.code == _1260 and self._ctl:  # device can be instatiated with a CTL
+        if msg.code == _1260 and self._ctl and not self._gwy.config.disable_sending:
+            # device can be instatiated with a CTL
             self._send_cmd(Command.get_dhw_temp(self._ctl.id))
 
     @property
@@ -1172,53 +1197,53 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     # _STATE = super().MODULATION_LEVEL
 
     SCHEMA_MSG_IDS = (
-        0x03,  # ..3: "Slave configuration",
-        0x06,  # ..6: "Remote boiler parameter flags",                      # 0x38, 0x39
-        0x0F,  # .15: "Max. boiler capacity (kW) and modulation level setting (%)",
-        0x30,  # .48: "DHW Setpoint upper & lower bounds for adjustment (°C)",
-        0x31,  # .49: "Max CH water Setpoint upper & lower bounds for adjustment (°C)",
-        0x7D,  # 125: "Opentherm version Slave",                            # not native
-        0x7F,  # 127: "Slave product version number and type",
+        "03",  # ..3: "Slave configuration",
+        "06",  # ..6: "Remote boiler parameter flags",                      # 0x38, 0x39
+        "0F",  # .15: "Max. boiler capacity (kW) and modulation level setting (%)",
+        "30",  # .48: "DHW Setpoint upper & lower bounds for adjustment (°C)",
+        "31",  # .49: "Max CH water Setpoint upper & lower bounds for adjustment (°C)",
+        "7D",  # 125: "Opentherm version Slave",                            # not native
+        "7F",  # 127: "Slave product version number and type",
     )
     PARAMS_MSG_IDS = (
-        0x38,  # .56: "DHW Setpoint (°C) (Remote parameter 1)",             # see: 0x06
-        0x39,  # .57: "Max CH water Setpoint (°C) (Remote parameter 2)",    # see: 0x06
+        "38",  # .56: "DHW Setpoint (°C) (Remote parameter 1)",             # see: 0x06
+        "39",  # .57: "Max CH water Setpoint (°C) (Remote parameter 2)",    # see: 0x06
         # These are error codes...
-        0x05,  # ..5: "Fault flags & OEM codes",
-        0x73,  # 115: "OEM diagnostic code",
+        "05",  # ..5: "Fault flags & OEM codes",
+        "73",  # 115: "OEM diagnostic code",
         # These are STATUS seen RQ'd by 01:/30:, but here to retreive less frequently
-        0x71,  # 113: "Number of un-successful burner starts",
-        0x72,  # 114: "Number of times flame signal was too low",
-        0x74,  # 116: "Number of starts burner",
-        0x75,  # 117: "Number of starts central heating pump",
-        0x76,  # 118: "Number of starts DHW pump/valve",
-        0x77,  # 119: "Number of starts burner during DHW mode",
-        0x78,  # 120: "Number of hours burner is in operation (i.e. flame on)",
-        0x79,  # 121: "Number of hours central heating pump has been running",
-        0x7A,  # 122: "Number of hours DHW pump has been running/valve has been opened",
-        0x7B,  # 123: "Number of hours DHW burner is in operation during DHW mode",
+        "71",  # 113: "Number of un-successful burner starts",
+        "72",  # 114: "Number of times flame signal was too low",
+        "74",  # 116: "Number of starts burner",
+        "75",  # 117: "Number of starts central heating pump",
+        "76",  # 118: "Number of starts DHW pump/valve",
+        "77",  # 119: "Number of starts burner during DHW mode",
+        "78",  # 120: "Number of hours burner is in operation (i.e. flame on)",
+        "79",  # 121: "Number of hours central heating pump has been running",
+        "7A",  # 122: "Number of hours DHW pump has been running/valve has been opened",
+        "7B",  # 123: "Number of hours DHW burner is in operation during DHW mode",
     )
     STATUS_MSG_IDS = (
-        0x00,  # ..0: "Master/Slave status flags",                          # not native
-        0x01,  # ..1: "CH water temperature Setpoint (°C)",                 # also R/W
-        0x11,  # .17: "Relative Modulation Level (%)",
-        0x12,  # .18: "Water pressure in CH circuit (bar)",
-        0x13,  # .19: "Water flow rate in DHW circuit. (L/min)",
-        0x19,  # .25: "Boiler flow water temperature (°C)",
-        0x1A,  # .26: "DHW temperature (°C)",
-        0x1B,  # .27: "Outside temperature (°C)",  # TODO: any value here?  # not native
-        0x1C,  # .28: "Return water temperature (°C)",
+        "00",  # ..0: "Master/Slave status flags",                          # not native
+        "01",  # ..1: "CH water temperature Setpoint (°C)",                 # also R/W
+        "11",  # .17: "Relative Modulation Level (%)",
+        "12",  # .18: "Water pressure in CH circuit (bar)",
+        "13",  # .19: "Water flow rate in DHW circuit. (L/min)",
+        "19",  # .25: "Boiler flow water temperature (°C)",
+        "1A",  # .26: "DHW temperature (°C)",
+        "1B",  # .27: "Outside temperature (°C)",  # TODO: any value here?  # not native
+        "1C",  # .28: "Return water temperature (°C)",
     )
-    WRITE_MSG_IDS = (  # Write-Data, some may also Read-Data (will need to check)
-        0x01,  # ..1:  -see above-
-        0x02,  # ..2: "Master configuration",
-        0x0E,  # .14: "Maximum relative modulation level setting (%)",  # c.f. 0x11
-        0x10,  # .16: "Room Setpoint (°C)",     # tell slave the room setpoint?
-        0x18,  # .24: "Room temperature (°C)",  # tell slave the room temp?
-        0x38,  # .56:  -see above-
-        0x39,  # .57:  -see above-
-        0x7C,  # 124: "Opentherm version Master",
-        0x7E,  # 126: "Master product version number and type",
+    WRITE_MSG_IDS = (  # Write-Data, NB: some are also Read-Data
+        "01",  # ..1:  -see above-
+        "02",  # ..2: "Master configuration",
+        "0E",  # .14: "Maximum relative modulation level setting (%)",  # c.f. 0x11
+        "10",  # .16: "Room Setpoint (°C)",     # tell slave the room setpoint?
+        "18",  # .24: "Room temperature (°C)",  # tell slave the room temp?
+        "38",  # .56:  -see above-
+        "39",  # .57:  -see above-
+        "7C",  # 124: "Opentherm version Master",
+        "7E",  # 126: "Master product version number and type",
     )
 
     def __init__(self, *args, **kwargs) -> None:
@@ -1231,7 +1256,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         self._supported_msg = {}
 
     def __repr__(self) -> str:
-        return f"{self.id} ({self._domain_id}): {self.modulation_level}"  # 3EF0
+        return f"{self.id} ({self._domain_id}): {self.rel_modulation_level}"  # 3EF0
 
     @discover_decorator
     def _discover(self, discover_flag=Discover.ALL) -> None:
@@ -1261,94 +1286,102 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
                 # "2401",
                 # "2410",
                 # "2420"
-                "0150",
-                "1098",
-                "10B0",
-                "3221",
-                "3223",
+                "0150",  # R8820A only?
+                "1098",  # R8820A only?
+                "10B0",  # R8820A only?
+                "3221",  # R8810A/20A
+                "3223",  # R8810A/20A
             ):
                 self._send_cmd(Command(RQ, code, "00", self.id))
 
         if discover_flag & Discover.STATUS:
-            self._send_cmd(Command(RQ, "12F0", "00", self.id))
+            self._send_cmd(Command(RQ, _12F0, "00", self.id))
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
                 for m in self.STATUS_MSG_IDS
                 if self._supported_msg.get(m) is not False
-                and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
-            ]
+                # nd (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
+            ]  # TODO: add expired
+
+        if discover_flag & Discover.STATUS and DEV_MODE:
+            for code in (_1081, _1300, _10A0, _1260, _1290, _22D9):
+                self._send_cmd(Command(RQ, code, "00", self.id))
 
     def _handle_msg(self, msg) -> None:
-        if msg.code == _3220 and msg._pkt.payload[4:] in ("121980", "1347AB", "1B47AB"):
-            if msg.payload[MSG_ID] not in self._supported_msg:
-                self._supported_msg[msg.payload[MSG_ID]] = None
-
-            elif self._supported_msg[msg.payload[MSG_ID]] is None:
-                self._supported_msg[msg.payload[MSG_ID]] = False
-                _LOGGER.warning(
-                    f"{msg._pkt} << OpenTherm: deprecating msg_id "
-                    f"{msg.payload[MSG_ID]}: it appears unsupported",
-                )
-
-        if msg.code == _3220 and not self._supported_msg.get(msg.payload[MSG_ID]):
-            return
-
         super()._handle_msg(msg)
 
-        if msg.code == _3220:
-            self._supported_msg[msg.payload[MSG_ID]] = msg.payload[MSG_TYPE] not in (
+        if msg.code != _3220:
+            return
+
+        msg_id = f"{msg.payload[MSG_ID]:02X}"
+
+        if msg._pkt.payload[4:] == "121980" or msg._pkt.payload[6:] == "47AB":
+            if msg_id not in self._supported_msg:
+                self._supported_msg[msg_id] = None
+
+            elif self._supported_msg[msg_id] is None:
+                self._supported_msg[msg_id] = False
+                _LOGGER.warning(
+                    f"{msg._pkt} << OpenTherm: deprecating msg_id "
+                    f"0x{msg_id}: it appears unsupported",
+                )
+
+        else:
+            self._supported_msg[msg_id] = msg.payload[MSG_TYPE] not in (
                 "Data-Invalid",
                 "Unknown-DataId",
                 "-reserved-",
             )
 
     def _ot_msg_value(self, msg_id) -> Optional[float]:
-        try:
-            return self._opentherm_msg[f"{msg_id:02X}"].payload[VALUE]
-        except KeyError:
-            return
+        if (
+            (msg := self._opentherm_msg.get(msg_id))
+            and self._supported_msg[msg_id]
+            and not msg._expired
+        ):
+            return msg.payload.get(VALUE)
 
     @property
     def boiler_output_temp(self) -> Optional[float]:  # 3220/19
-        return self._ot_msg_value(0x19)
+        return self._ot_msg_value("19")
 
     @property
     def boiler_return_temp(self) -> Optional[float]:  # 3220/1C
-        return self._ot_msg_value(0x1C)
+        return self._ot_msg_value("1C")
 
     @property
     def boiler_setpoint(self) -> Optional[float]:  # 3220/01 (22D9)
         # return self._msg_value(_22D9, key=self.BOILER_SETPOINT)
-        return self._ot_msg_value(0x01)
+        return self._ot_msg_value("01")
 
     @property
     def ch_max_setpoint(self) -> Optional[float]:  # 3220/39 (1081)
-        return self._ot_msg_value(0x39)
+        return self._ot_msg_value("39")
 
     @property
     def ch_water_pressure(self) -> Optional[float]:  # 3220/12 (1300)
-        return self._ot_msg_value(0x12)
+        return self._ot_msg_value("12")
 
     @property
     def dhw_flow_rate(self) -> Optional[float]:  # 3220/13
-        return self._ot_msg_value(0x13)
+        return self._ot_msg_value("13")
 
     @property
     def dhw_setpoint(self) -> Optional[float]:  # 3220/38 (10A0)
-        return self._ot_msg_value(0x89)
+        return self._ot_msg_value("38")
 
     @property
     def dhw_temp(self) -> Optional[float]:  # 3220/1A (1260)
-        return self._ot_msg_value(0x1A)
+        return self._ot_msg_value("1A")
 
     @property
     def outside_temp(self) -> Optional[float]:  # 3220/1B (1290)
-        return self._ot_msg_value(0x1B)
+        return self._ot_msg_value("1B")
 
     @property
     def rel_modulation_level(self) -> Optional[float]:  # 3220/11 (3EFx)
         # return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
-        return self._ot_msg_value(0x11)
+        return self._ot_msg_value("11")
 
     @staticmethod
     def _msg_name(msg) -> str:
@@ -1409,9 +1442,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         return {
             **super().schema,
             "known_msg_ids": {
-                f"{k:02X}": OPENTHERM_MESSAGES[k].get("var", f"{k:02X}")
-                if k in OPENTHERM_MESSAGES
-                else f"{k:02X}"
+                k: OPENTHERM_MESSAGES[int(k, 16)].get("var", k)
                 for k, v in sorted(self._supported_msg.items())
                 if v
             },
@@ -1792,19 +1823,7 @@ def create_device(gwy, dev_id, dev_class=None, **kwargs) -> Device:
 
     device = DEVICE_BY_CLASS.get(dev_class, Device)(gwy, dev_addr, **kwargs)
 
-    if not gwy.serial_port:
-        return device
-
-    gwy._add_task(
-        device._discover, discover_flag=Discover.SCHEMA, delay=0, period=86400
-    )
-
-    delay = randint(10, 20)
-    gwy._add_task(
-        device._discover, discover_flag=Discover.PARAMS, delay=delay, period=21600
-    )  # 21600 = 60*60*6
-    gwy._add_task(
-        device._discover, discover_flag=Discover.STATUS, delay=delay + 3, period=60
-    )
+    if isinstance(gwy.pkt_protocol, PacketProtocolPort):
+        device._start_discovery()
 
     return device
