@@ -6,7 +6,7 @@
 import logging
 from typing import List
 
-from .const import DISCOVER_ALL
+from .const import Discover
 
 from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
 
@@ -21,19 +21,12 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def discovery_filter(func):
-    """Bypass discovery if not configured to do so, or if unlikely to get an RP."""
+def discover_decorator(func):
+    def wrapper(self, discover_flag=None) -> None:
 
-    def wrapper(self, *args, **kwargs) -> None:
-        if any(
-            (
-                self._gwy.config.disable_sending,
-                self._gwy.config.disable_discovery,
-                getattr(self, "has_battery", None),
-            )
-        ):
+        if self._gwy.config.disable_discovery:
             return
-        return func(self, *args, **kwargs)
+        return func(self, discover_flag=discover_flag)
 
     return wrapper
 
@@ -53,7 +46,7 @@ class Entity:
         self._msgs = {}
         self._msgz = {}
 
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         pass
 
     def _handle_msg(self, msg) -> None:  # TODO: beware, this is a mess
@@ -92,14 +85,27 @@ class Entity:
     #     """Return a flattened version of ..."""
     #     return {msg.dtm: msg._pkt for msg in self._msgs_db}
 
-    def _send_cmd(self, code, dest_id, payload, verb=RQ, **kwargs) -> None:
-        self._msgs.pop(code, None)  # TODO: remove old one, so we can tell if RP'd rcvd
-        self._gwy.send_cmd(self._gwy.create_cmd(verb, dest_id, code, payload, **kwargs))
-        # was: self._gwy.send_cmd(Command(verb, code, payload, dest_id, **kwargs))
+    def _make_cmd(self, code, dest_id, payload, verb=RQ, **kwargs) -> None:
+        self._send_cmd(self._gwy.create_cmd(verb, dest_id, code, payload, **kwargs))
 
-    def _msg_value(
-        self, code, verb=None, key=None, zone_idx=None, domain_id=None
-    ) -> dict:
+    def _send_cmd(self, cmd, **kwargs) -> None:
+        if self._gwy.config.disable_sending:
+            _LOGGER.warning(f"{cmd} < Sending is disabled")
+            return
+
+        # if getattr(self, "has_battery", None):
+        #     return
+
+        self._msgs.pop(cmd.code, None)  # TODO: remove, so we can tell if RP'd rcvd
+        self._gwy.send_cmd(cmd)
+
+    def _msg_value(self, code, *args, **kwargs) -> dict:
+
+        if isinstance(code, (str, tuple)):  # a code or a tuple of codes
+            return self._msg_value_code(code, *args, **kwargs)
+        return self._msg_value_msg(code, *args, **kwargs)  # assume is a Message
+
+    def _msg_value_code(self, code, verb=None, key=None, **kwargs) -> dict:
 
         assert (
             not isinstance(code, tuple) or verb is None
@@ -118,6 +124,10 @@ class Entity:
         else:
             msg = self._msgs.get(code)
 
+        return self._msg_value_msg(msg, key=key, **kwargs)
+
+    def _msg_value_msg(self, msg, key=None, zone_idx=None, domain_id=None) -> dict:
+
         if msg is None:
             return
         elif msg._expired:
