@@ -490,7 +490,7 @@ class Command(PacketBase):
 
     @classmethod  # constructor for RQ/3220
     @validate_command()
-    def get_opentherm_data(cls, dev_id: str, msg_id: int, **kwargs):
+    def get_opentherm_data(cls, dev_id: str, msg_id: Union[int, str], **kwargs):
         """Constructor to get (Read-Data) opentherm msg value (c.f. parser_3220)."""
         msg_id = msg_id if isinstance(msg_id, int) else int(msg_id, 16)
         payload = f"0080{msg_id:02X}0000" if parity(msg_id) else f"0000{msg_id:02X}0000"
@@ -881,7 +881,8 @@ class Command(PacketBase):
         """
         #  I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
 
-        assert dev_id[:2] in ("03", "12", "22", "34")  # dex
+        if dev_id[:2] not in ("03", "12", "22", "34"):  # dex
+            raise TypeError(f"Wrong device type: {dev_id[:2]}, try 03, 12, 22, 34")
 
         payload = f"00{temp_to_hex(temperature)}"
         return cls.packet(I_, _30C9, payload, addr0=dev_id, addr2=dev_id, **kwargs)
