@@ -164,7 +164,11 @@ class Packet(PacketBase):
         """Return fraction used of the normal lifetime of packet.
 
         A packet is 'expired' when >1.0, and should be tombstoned when >2.0. Returns
-        False if the packet does not expire.
+        False if the packet does not expire (or None is N/A?).
+
+        NB: this is only the fact if the packet has expired, or not. Any opinion to
+        whether it *matters* that the packet has expired, is up to higher layers of the
+        stack.
         """
 
         if self._timeout is None:
@@ -256,13 +260,14 @@ def pkt_timeout(pkt) -> Optional[td]:  # NOTE: import OtbGateway ??
         from ..devices import OtbGateway  # to prevent circular references
 
         if pkt.payload[4:6] in OtbGateway.SCHEMA_MSG_IDS:
-            timeout = None
+            return
+        # elif pkt.payload[4:6] in OtbGateway.WRITE_MSG_IDS and Write-Date:
+        #     timeout = td(seconds=3)
         elif pkt.payload[4:6] in OtbGateway.PARAMS_MSG_IDS:
+            # NB: includes "Number of starts burner" and similars
             timeout = td(minutes=60)
-        # elif pkt.payload[4:6] in OtbGateway.STATUS_MSG_IDS:
-        #     timeout = td(minutes=5)
-        else:
-            timeout = td(minutes=5)
+        else:  # elif pkt.payload[4:6] in OtbGateway.STATUS_MSG_IDS:
+            timeout = td(minutes=3)
 
     # elif pkt.code in (_3B00, _3EF0, ):  # TODO: 0008, 3EF0, 3EF1
     #     timeout = td(minutes=6.7)  # TODO: WIP
