@@ -7,6 +7,7 @@
 # - Evsdd: 0404
 # - Ierlandfan: 3150, 31D9, 31DA, others
 # - ReneKlootwijk: 3EF0
+# - brucemiranda: 3EF0, others
 
 import logging
 from datetime import datetime as dt
@@ -14,6 +15,7 @@ from datetime import timedelta as td
 from typing import Any, Optional, Union
 
 from .const import (
+    _000C_DEVICE,
     _000C_DEVICE_TYPE,
     _0005_ZONE_TYPE,
     _0418_DEVICE_CLASS,
@@ -104,6 +106,13 @@ from .const import (  # noqa: F401, isort: skip
     _PUZZ,
 )
 
+LOOKUP_PUZZ = {
+    "10": "engine",  # .    # version str, e.g. v0.14.0
+    "11": "impersonating",  # pkt header, e.g. 30C9| I|03:123001 (15 characters, packed)
+    "12": "message",  # .   # message only, max len is 16 ascii characters
+    "13": "message",  # .   # message only, but without a timestamp, max len 22 chars
+}  # "00" is reserved
+
 DEV_MODE = __dev_mode__ and False
 TEST_MODE = False  # enable to test constructors (usu. W)
 
@@ -198,19 +207,25 @@ def parser_0005(payload, msg) -> Optional[dict]:  # TODO: needs a cleanup
     #  I --- 34:064023 --:------ 34:064023 0005 012 000A0000-000F0000-00100000
 
     def _parser(seqx) -> dict:
+        if msg.src.type == "02":  # seqx[2:4] == _0005_ZONE.UFH:
+            zone_mask = flag8(seqx[6:8], lsb=True)
+        else:
+            zone_mask = flag8(seqx[4:6], lsb=True) + flag8(seqx[6:8], lsb=True)
         return {
-            "_zone_type": seqx[2:4],
-            "zone_mask": (flag8(seqx[4:6], lsb=True) + flag8(seqx[6:8], lsb=True)),
+            "_device_class": seqx[2:4],
+            "zone_mask": zone_mask,
             "zone_type": _0005_ZONE_TYPE.get(seqx[2:4], f"unknown_{seqx[2:4]}"),
         }
 
     if msg._has_array:
-        assert msg.verb == I_ and msg.src.type == "34", f"{msg._pkt} # expecting I/34:"
+        assert (
+            msg.verb == I_ and msg.src.type == "34"
+        ), f"{msg._pkt} # expecting I/34:"  # DEX
         return [_parser(payload[i : i + 8]) for i in range(0, len(payload), 8)]
 
     if msg.verb == RQ:  # RQs have a context: zone_type
         return {
-            "_zone_type": payload[2:4],
+            "_device_class": payload[2:4],
             "zone_type": _0005_ZONE_TYPE.get(payload[2:4], f"unknown_{payload[2:4]}"),
         }
 
@@ -243,7 +258,7 @@ def parser_0008(payload, msg) -> Optional[dict]:
     # https://www.domoticaforum.eu/viewtopic.php?f=7&t=5806&start=105#p73681
     # e.g. Electric Heat Zone
 
-    if msg.src.type == "31" and msg.len == 13:  # Honeywell Japser ?HVAC
+    if msg.src.type == "31" and msg.len == 13:  # Honeywell Japser ?HVAC, DEX
         assert msg.len == 13, "expecting length 13"
         return {
             "ordinal": f"0x{payload[2:8]}",
@@ -340,20 +355,20 @@ def parser_000c(payload, msg) -> Optional[dict]:
 
     def complex_index(seqx, msg) -> dict:  # complex index
         # TODO: 000C to a UFC should be ufh_ifx, not zone_idx
-        if msg.src.type == "02":
+        if msg.src.type == "02":  # DEX
             assert int(seqx, 16) < 8, f"invalid ufh_idx: '{seqx}' (0x00)"
             return {
                 "ufh_idx": seqx,
                 "zone_id": None if payload[4:6] == "7F" else payload[4:6],
             }
 
-        if payload[2:4] in ("0D", "0E"):  # ("000D", _000E, "010E")
+        if payload[2:4] in (_000C_DEVICE.DHW_SENSOR, _000C_DEVICE.DHW):
             assert (
                 int(seqx, 16) < 1 if payload[2:4] == "0D" else 2
             ), f"invalid _idx: '{seqx}' (0x01)"
             return {"domain_id": "FA"}
 
-        if msg.raw_payload[2:4] == "0F":
+        if payload[2:4] == _000C_DEVICE.HTG:
             assert int(seqx, 16) < 1, f"invalid _idx: '{seqx}' (0x02)"
             return {"domain_id": "FC"}
 
@@ -369,13 +384,13 @@ def parser_000c(payload, msg) -> Optional[dict]:
         return {hex_id_to_dec(seqx[6:12]): seqx[4:6]}
 
     device_class = _000C_DEVICE_TYPE.get(payload[2:4], f"unknown_{payload[2:4]}")
-    if device_class == ATTR_DHW_VALVE and msg.raw_payload[:2] == "01":
+    if device_class == ATTR_DHW_VALVE and payload[:2] == "01":
         device_class = ATTR_DHW_VALVE_HTG
 
     result = {
         **complex_index(payload[:2], msg),
-        "device_class": device_class,
         "_device_class": payload[2:4],
+        "device_class": device_class,
     }
     if msg.verb == RQ:  # RQs have a context: index, zone_type
         return result
@@ -483,22 +498,13 @@ def parser_0404(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # system_fault
 def parser_0418(payload, msg) -> Optional[dict]:
-    # RP --- 01:145038 18:013393 --:------ 0418 022 000002B0061C040000008D9466277FFFFF7000367F95  # noqa
+    # RP --- 01:145038 18:013393 --:------ 0418 022 000000B0000000000000000000007FFFFF7000000000  # noqa
     # RP --- 01:145038 18:013393 --:------ 0418 022 000036B0010000000000108000007FFFFF7000000000  # noqa
     # RP --- 01:145038 18:013393 --:------ 0418 022 000000B00401010000008694A3CC7FFFFF70000ECC8A  # noqa
-    # RP --- 01:145038 18:013393 --:------ 0418 022 00C001B004010100000086949BCB7FFFFF70000ECC8A  # noqa
-    # RP --- 01:145038 18:013393 --:------ 0418 022 000000B0000000000000000000007FFFFF7000000000  # noqa
+    #  I --- 01:037519 --:------ 01:037519 0418 022 000000B0050000000000239581877FFFFF7000000001  # Evotouch Battery Error  # noqa
     # RP --- 01:037519 18:140805 --:------ 0418 022 004024B0060006000000CB94A112FFFFFF70007AD47D  # noqa
-
-    # 00-4000B0-061C04-0000-008F14B0DB7F-FFFF7000-367F95
-    # 00-0000B0-040101-0000-008694A3CC7F-FFFF7000-0ECC8A
-    # 00-C001B0-040101-0000-0086949BCB7F-FFFF7000-0ECC8A
-    # 00-0002B0-061C04-0000-008D9466277F-FFFF7000-367F95
-    # 00-4024B0-060006-0000-00CB94A112FF-FFFF7000-7AD47D
-    # 00-0036B0-010000-0000-00108000007F-FFFF7000-000000
-    # 00-0000B0-000000-0000-00000000007F-FFFF7000-000000
-    #    0      0      1    1            3        3
-    #    2      8      4    8            0        8
+    #                                                 0     0   1     1            3        3
+    #                                                 2     8   2     8            0        8
 
     # assert int(payload[4:6], 16) < 64, f"Unexpected log_idx: 0x{payload[4:6]}"
 
@@ -547,15 +553,20 @@ def parser_0418(payload, msg) -> Optional[dict]:
     return {"log_entry": [v for k, v in result.items() if k != "log_idx"]}
 
 
-@parser_decorator  # unknown, from STA
+@parser_decorator  # unknown, from STA, VMS
 def parser_042f(payload, msg) -> Optional[dict]:
-    # 055  I --- 34:064023 --:------ 34:064023 042F 008 00000000230023F5
-    # 063  I --- 34:064023 --:------ 34:064023 042F 008 00000000240024F5
-    # 049  I --- 34:064023 --:------ 34:064023 042F 008 00000000250025F5
-    # 045  I --- 34:064023 --:------ 34:064023 042F 008 00000000260026F5
-    # 045  I --- 34:092243 --:------ 34:092243 042F 008 0000010021002201
-    # 000  I     34:011469 --:------ 34:011469 042F 008 00000100030004BC
-    # 000  I --- 32:168090 --:------ 32:168090 042F 009 000000100F00105050 - ???
+    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0023-0023-F5
+    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0024-0024-F5
+    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0025-0025-F5
+    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0026-0026-F5
+    #  I --- 34:092243 --:------ 34:092243 042F 008 00-0001-0021-0022-01
+    #  I     34:011469 --:------ 34:011469 042F 008 00-0001-0003-0004-BC
+
+    #  I --- 32:168090 --:------ 32:168090 042F 009 00-0000100F00105050
+    #  I --- 32:166025 --:------ 32:166025 042F 009 00-050E0B0C00111470
+
+    if msg.len != 8:
+        return {"unknown": payload[2:]}
 
     return {
         "counter_1": int(payload[2:6], 16),
@@ -632,6 +643,11 @@ def parser_1060(payload, msg) -> Optional[dict]:
     }
 
 
+@parser_decorator  # max_ch_setpoint (supply high limit)
+def parser_1081(payload, msg) -> Optional[dict]:
+    return {"setpoint": temp_from_hex(payload[2:])}
+
+
 @parser_decorator  # unknown (non-Evohome, e.g. ST9520C)
 def parser_1090(payload, msg) -> dict:
     # 14:08:05.176 095 RP --- 23:100224 22:219457 --:------ 1090 005 007FFF01F4
@@ -647,10 +663,11 @@ def parser_1090(payload, msg) -> dict:
     }
 
 
-@parser_decorator  # dhw_params  # FIXME: a bit messy
+@parser_decorator  # dhw (cylinder) params  # FIXME: a bit messy
 def parser_10a0(payload, msg) -> Optional[dict]:
     # RQ --- 07:045960 01:145038 --:------ 10A0 006 00-1087-00-03E4  # RQ/RP, every 24h
     # RP --- 01:145038 07:045960 --:------ 10A0 006 00-109A-00-03E8
+    # RP --- 10:048122 18:006402 --:------ 10A0 003 00-1B58
 
     # these may not be reliable...
     # RQ --- 01:136410 10:067219 --:------ 10A0 002 0000
@@ -703,51 +720,76 @@ def parser_10a0(payload, msg) -> Optional[dict]:
 def parser_10e0(payload, msg) -> Optional[dict]:
     assert msg.len >= 19, msg.len  # in (19, 28, 30, 36, 38), msg.len
 
-    DEV_MODE = False
-    if not DEV_MODE:
-        pass
-    elif msg.src.type == "01":
-        assert payload[2:20] in (
-            "0002FF0119FFFFFFFF",
-            "0002FF0163FFFFFFFF",
-        ), payload[2:20]
-    elif msg.src.type == "02":
-        assert payload[2:20] in ("0003FF0203FFFF0001",), payload[2:20]
-    elif msg.src.type == "04":
-        assert payload[2:20] in (
-            "0002FF0412FFFFFFFF",
-            "0002FF050BFFFFFFFF",
-        ), payload[2:20]
-    elif msg.src.type == "08":
-        assert payload[2:20] in ("0002FF0802FFFFFFFE",), payload[2:20]
-    elif msg.src.type == "10":
-        assert payload[2:20] in (
-            "0001C8810B0700FEFF",
-            "0002FF0A0CFFFFFFFF",
-        ), payload[2:20]
-    elif msg.src.type == "20":
-        assert payload[2:20] in (
-            "000100140C06010000",
-            "0001001B190B010000",
-            "0001001B221201FEFF",
-            "0001001B271501FEFF",
-            "0001001B281501FEFF",
-        ), payload[2:20]
-    elif msg.src.type == "30":
-        assert payload[2:20] in (
-            "0001C90011006CFEFF",
-            "0002FF1E01FFFFFFFF",
-            "0002FF1E03FFFFFFFF",
-        ), payload[2:20]
-    elif msg.src.type == "31":
-        assert payload[2:20] in ("0002FF1F02FFFFFFFF",), payload[2:20]
-    elif msg.src.type == "32":
-        assert payload[2:20] in ("0001C85802016CFFFF",), payload[2:20]
-    elif msg.src.type == "34":
-        assert payload[2:20] in (
-            "0001C8380A0100F1FF",
-            "0001C8380F0100F1FF",
-        ), payload[2:20]
+    # if DEV_MODE:
+    try:
+        if msg.src.type == "01":  # DEX
+            assert payload[2:20] in (
+                "0002FF0119FFFFFFFF",  # EvoTouch Colour
+                "0002FF0163FFFFFFFF",  # Evo Color
+            ), payload[2:20]
+        elif msg.src.type == "02":  # DEX
+            assert payload[2:20] in (
+                "0003FF0203FFFF0001",  # HCE80 V3.10 061117
+            ), payload[2:20]
+        elif msg.src.type == "04":  # DEX
+            assert payload[2:20] in (
+                "0002FF0412FFFFFFFF",  # HR92 Radiator Ctrl.
+                "0002FF050BFFFFFFFF",  # HR91 Radiator Ctrl.
+            ), payload[2:20]
+        elif msg.src.type == "08":  # DEX
+            assert payload[2:20] in (
+                "0002FF0802FFFFFFFE",  # Jasper EIM (non-evohome)
+            ), payload[2:20]
+        elif msg.src.type == "10":  # DEX
+            assert payload[2:20] in (
+                "0001C8810B0700FEFF",  # R8820A
+                "0002FF0A0CFFFFFFFF",  # R8810A
+            ), payload[2:20]
+        elif msg.src.type == "20":  # DEX
+            assert payload[2:20] in (
+                "000100140C06010000",  # n/a
+                "0001001B190B010000",  # n/a
+                "0001001B221201FEFF",  # CVE-RF
+                "0001001B271501FEFF",  # CVE-RF
+                "0001001B281501FEFF",  # CVE-RF
+            ), payload[2:20]
+        elif msg.src.type == "30":  # DEX
+            assert payload[2:20] in (
+                "0001C90011006CFEFF",  # BRDG-02JAS01 (fan, PIV)
+                "0002FF1E01FFFFFFFF",  # Internet Gateway
+                "0002FF1E03FFFFFFFF",  # Internet Gateway
+            ), payload[2:20]
+        elif msg.src.type == "31":  # DEX
+            assert payload[2:20] in (
+                "0002FF1F02FFFFFFFF",  # Jasper Stat TXXX
+            ), payload[2:20]
+        elif msg.src.type == "32":  # DEX
+            # VMN-23LMH23 (switch, 4-button)
+            assert payload[2:20] in (
+                "0001C85701016CFFFF",  # VMS-23C33   (sensor, CO2)
+                "0001C85802016CFFFF",  # VMS-23HB33  (sensor, RH/temp)
+                "0001C85803016CFFFF",  # VMS-23HB33  (sensor, RH/temp)
+                "0001C8950B0A67FEFF",  # VMD-15RMS86 (fan, Orcon HRC 500)
+            ), payload[2:20]
+        elif msg.src.type == "34":  # DEX
+            assert payload[2:20] in (
+                "0001C8380A0100F1FF",  # T87RF2025
+                "0001C8380F0100F1FF",  # T87RF2025
+            ), payload[2:20]
+        elif msg.src.type == "37":  # DEX
+            assert payload[2:20] in (
+                "0001001B2E1901FEFF",  # CVE-RF
+                "0001001B311901FEFF",  # CVE-RF
+                "0001001B361B01FEFF",  # CVE-RF
+                "0001001B381B01FEFF",  # CVE-RF
+                "00010028080101FEFF",  # VMS-12C39
+            ), payload[2:20]
+
+    except AssertionError:
+        _LOGGER.warning(
+            f"{msg._pkt} < Support development by reporting this pkt, "
+            "please include a description/the make & model of this device"
+        )
 
     date_2 = date_from_hex(payload[20:28])  # could be 'FFFFFFFF'
     date_1 = date_from_hex(payload[28:36])  # could be 'FFFFFFFF'
@@ -762,12 +804,17 @@ def parser_10e0(payload, msg) -> Optional[dict]:
     }
 
 
+@parser_decorator  # device_id
+def parser_10e1(payload, msg) -> Optional[dict]:
+    return {"device_id": hex_id_to_dec(payload[2:])}
+
+
 @parser_decorator  # tpi_params (domain/zone/device)  # FIXME: a bit messy
 def parser_1100(payload, msg) -> Optional[dict]:
     def complex_index(seqx) -> dict:
         return {"domain_id": seqx} if seqx[:1] == "F" else {}  # only FC
 
-    if msg.src.type == "08":  # Honeywell Japser ?HVAC
+    if msg.src.type == "08":  # Honeywell Japser ?HVAC, DEX
         assert msg.len == 19, msg.len
         return {
             "ordinal": f"0x{payload[2:8]}",
@@ -825,21 +872,49 @@ def parser_1100(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # dhw_temp
+@parser_decorator  # dhw cylinder temperature
 def parser_1260(payload, msg) -> Optional[dict]:
     return {"temperature": temp_from_hex(payload[2:])}
 
 
-@parser_decorator  # outdoor_temp
+@parser_decorator  # outdoor humidity
+def parser_1280(payload, msg) -> Optional[dict]:
+    # this packet never seen in the wild
+    # assert msg.len == 6 if type == ?? else 2, msg.len
+    assert payload[:2] == "00", payload[:2]  # domain?
+
+    rh = int(payload[2:4], 16) / 100 if payload[2:4] != "EF" else None
+    if msg.len == 2:
+        return {"relative_humidity": rh}
+
+    assert msg.len == 6, f"pkt length is {msg.len}, expected 6"
+    return {
+        "relative_humidity": rh,
+        "temperature": temp_from_hex(payload[4:8]),
+        "dewpoint_temp": temp_from_hex(payload[8:12]),
+    }
+
+
+@parser_decorator  # outdoor temperature
 def parser_1290(payload, msg) -> Optional[dict]:
     # evohome responds to an RQ
     return {"temperature": temp_from_hex(payload[2:])}
 
 
-@parser_decorator  # hvac_1298
+@parser_decorator  # co2_level
 def parser_1298(payload, msg) -> Optional[dict]:
     #  I --- 37:258565 --:------ 37:258565 1298 003 0007D0
-    return {"unknown": double(payload[2:])}
+    FAULT_CODES = {
+        "80": "sensor short circuit",
+        "81": "sensor open",
+        "83": "sensor value too high",
+        "84": "sensor value too low",
+        "85": "sensor unreliable",
+    }
+    if fault := FAULT_CODES.get(payload[:2]):
+        return {"sensor_fault": fault}
+
+    return {"co2_level": double(payload[2:])}
 
 
 @parser_decorator  # indoor_humidity (Nuaire RH sensor)
@@ -847,27 +922,32 @@ def parser_12a0(payload, msg) -> Optional[dict]:
     # assert msg.len == 6 if type == ?? else 2, msg.len
     assert payload[:2] == "00", payload[:2]  # domain?
 
-    RHUM_STATE = {
-        "EF": "not available",
-        "F0": "sensor short circuit",
-        "F1": "sensor open",
-        "F2": "not available",
-        "F3": "sensor value too high",
-        "F4": "sensor value too low",
-        "F5": "sensor unreliable",
-    }  # EFFF = N/A
-    assert payload[2:4] in RHUM_STATE or int(payload[2:4], 16) <= 100
+    FAULT_CODES_RHUM = {
+        "EF": "RH sensor not available ",
+        "F0": "RH sensor short circuit ",
+        "F1": "RH sensor open ",
+        "F2": "RH sensor not available",
+        "F3": "RH sensor value too high ",
+        "F4": "RH sensor value too low ",
+        "F5": "RH sensor unreliable ",
+    }  # relative humidity sensor
 
-    # TEMP_STATE = {
-    #     "7F": "not available",
+    assert payload[2:4] in FAULT_CODES_RHUM or int(payload[2:4], 16) <= 100
+    if fault := FAULT_CODES_RHUM.get(payload[2:4]):
+        return {"sensor_fault": fault}
+
+    # FAULT_CODES_TEMP = {
+    #     "7F": "sensor not available",
     #     "80": "sensor short circuit",
     #     "81": "sensor open",
-    #     "82": "not available",
+    #     "82": "sensor not available",
     #     "83": "sensor value too high",
     #     "84": "sensor value too low",
     #     "85": "sensor unreliable",
-    # }  # 7F = N/A, same for 1298
-    # assert payload[4:6] in TEMP_STATE or ...
+    # }
+    # assert payload[4:6] in FAULT_CODES_TEMP or ...
+    # if (fault := FAULT_CODES_TEMP.get(payload[2:4])):
+    #     return {"sensor_fault": fault}
 
     rh = int(payload[2:4], 16) / 100 if payload[2:4] != "EF" else None
     if msg.len == 2:
@@ -890,13 +970,20 @@ def parser_12b0(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # displayed_temp (on a TR87RF bound to a RFG100)
+@parser_decorator  # displayed temperature (on a TR87RF bound to a RFG100)
 def parser_12c0(payload, msg) -> Optional[dict]:
-    assert payload[:2] == "00", f"expecting 00, not {payload[:2]}"
-    assert payload[4:] == "01", f"expecting 01, not {payload[4:]}"
 
-    temp = None if payload[2:4] == "80" else int(payload[2:4], 16) / 2
-    return {"temperature": temp}
+    if payload[2:4] == "80":
+        temp = None
+    elif payload[4:] == "00":  # units are 1.0 F
+        temp = int(payload[2:4], 16)
+    else:  # if payload[4:] == "01":  # units are 0.5 C
+        temp = int(payload[2:4], 16) / 2
+
+    return {
+        "temperature": temp,
+        "units": {"00": "Fahrenheit", "01": "Celsius"}[payload[4:]],
+    }
 
 
 @parser_decorator  # hvac_12C8
@@ -906,6 +993,11 @@ def parser_12c8(payload, msg) -> Optional[dict]:
     return {"unknown": percent(payload[4:])}
 
 
+@parser_decorator  # ch_pressure
+def parser_1300(payload, msg) -> Optional[dict]:
+    return {"pressure": temp_from_hex(payload[2:])}  # is 2's complement still
+
+
 @parser_decorator  # system_sync
 def parser_1f09(payload, msg) -> Optional[dict]:
     # 22:51:19.287 067  I --- --:------ --:------ 12:193204 1F09 003 010A69
@@ -957,6 +1049,10 @@ def parser_1f41(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # rf_bind
 def parser_1fc9(payload, msg) -> list:
+    #  I is missing?
+    #  W --- 10:048122 01:145038 --:------ 1FC9 006 003EF028BBFA
+    #  I --- 01:145038 10:048122 --:------ 1FC9 006 00FFFF06368E
+
     #  I --- 07:045960 --:------ 07:045960 1FC9 012 0012601CB388001FC91CB388
     #  W --- 01:145038 07:045960 --:------ 1FC9 006 0010A006368E
     #  I --- 07:045960 01:145038 --:------ 1FC9 006 0012601CB388
@@ -1022,10 +1118,6 @@ def parser_1fca(payload, msg) -> list:
 
 @parser_decorator  # opentherm_sync, otb_sync
 def parser_1fd4(payload, msg) -> Optional[dict]:
-    assert msg.verb == I_, msg.verb
-    assert msg.len == 3, msg.len
-    assert payload[:2] == "00", payload[:2]
-
     return {"ticker": int(payload[2:], 16)}
 
 
@@ -1088,9 +1180,9 @@ def parser_22d0(payload, msg) -> Optional[dict]:
     return {"unknown": payload[2:]}
 
 
-@parser_decorator  # boiler_setpoint
+@parser_decorator  # desired boiler setpoint
 def parser_22d9(payload, msg) -> Optional[dict]:
-    return {"boiler_setpoint": temp_from_hex(msg.raw_payload[2:6])}
+    return {"setpoint": temp_from_hex(payload[2:6])}
 
 
 @parser_decorator  # switch_mode
@@ -1109,7 +1201,7 @@ def parser_22f1(payload, msg) -> Optional[dict]:
 
     if bitmap in FanSwitch.FAN_MODES:
         _action = {FanSwitch.FAN_MODE: FanSwitch.FAN_MODES[bitmap]}
-    elif bitmap in {9, 10}:
+    elif bitmap in {9, 10}:  # 00010001, 00010010
         _action = {FanSwitch.HEATER_MODE: FanSwitch.HEATER_MODES[bitmap]}
     else:
         _action = {}
@@ -1123,15 +1215,24 @@ def parser_22f1(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # switch_boost
 def parser_22f3(payload, msg) -> Optional[dict]:
-    from ..devices import FanSwitch  # FIXME: remove cyclic reference
-
     # NOTE: for boost timer for high
-    assert msg.len == 3, msg.len
     assert payload[:2] == "00", payload[:2]  # has no domain
     assert payload[2:4] == "00", payload[2:4]
-    assert payload[4:6] in ("0A", "14", "1E"), payload[4:6]
+    assert payload[4:6] in ("0A", "14", "1E"), payload[4:6]  # 10, 20, 30
 
-    return {FanSwitch.BOOST_TIMER: int(payload[4:6], 16)}
+    if msg.len >= 3:
+        result = {
+            "mode": "boost_timer",  # payload[2:4]
+            "minutes": int(payload[4:6], 16),
+        }
+
+    if msg.len >= 5:
+        result.update(parser_22f1(f"00{payload[6:10]}"))  # NOTE: a guess
+
+    if msg.len >= 7:
+        result.update({"_unknown": payload[10:]})
+
+    return result
 
 
 @parser_decorator  # setpoint (of device/zones)
@@ -1258,7 +1359,7 @@ def parser_2e04(payload, msg) -> Optional[dict]:
     return result
 
 
-@parser_decorator  # temperature (of device, zone/s)
+@parser_decorator  # current temperature (of device, zone/s)
 def parser_30c9(payload, msg) -> Optional[dict]:
 
     if msg._has_array:
@@ -1310,14 +1411,9 @@ def parser_313f(payload, msg) -> Optional[dict]:
     # https://www.automatedhome.co.uk/vbulletin/showthread.php?5085-My-HGI80-equivalent-Domoticz-setup-without-HGI80&p=36422&viewfull=1#post36422
     # every day at ~4am TRV/RQ->CTL/RP, approx 5-10secs apart (CTL respond at any time)
 
-    if msg.src.type == "01":
-        assert payload[2:4] in ("F0", "FC"), payload[2:4]
-    elif msg.src.type in ("12", "22"):
-        assert payload[2:4] == "38", payload[2:4]
-    elif msg.src.type == "30":
-        assert payload[2:4] == "60", payload[2:4]
-    else:
-        assert False, payload[2:4]
+    assert msg.src.type != "01" or payload[2:4] in ("F0", "FC"), payload[2:4]  # DEX
+    assert msg.src.type not in ("12", "22") or payload[2:4] == "38", payload[2:4]  # DEX
+    assert msg.src.type != "30" or payload[2:4] == "60", payload[2:4]  # DEX
 
     result = {
         "datetime": dtm_from_hex(payload[4:18]),
@@ -1346,7 +1442,7 @@ def parser_3150(payload, msg) -> Union[list, dict, None]:
 
     def complex_index(seqx, msg) -> dict:
         # assert seqx[:2] == "FC" or (int(seqx[:2], 16) < MAX_ZONES)  # <5, 8 for UFC
-        idx_name = "ufx_idx" if msg.src.type == "02" else "zone_idx"
+        idx_name = "ufx_idx" if msg.src.type == "02" else "zone_idx"  # DEX
         return {"domain_id" if seqx[:1] == "F" else idx_name: seqx[:2]}
 
     if msg._has_array:
@@ -1363,6 +1459,7 @@ def parser_3150(payload, msg) -> Union[list, dict, None]:
 
 @parser_decorator  # ventilation state
 def parser_31d9(payload, msg) -> Optional[dict]:
+    # NOTE: I have a suspicion that Itho use 0x00-C8 for %, whilst Nuaire use 0x00-64
     assert payload[:2] in ("00", "01", "21"), payload[2:4]
     assert payload[2:4] in ("00", "06", "80"), payload[2:4]
     assert payload[4:6] == "FF" or int(payload[4:6], 16) <= 200, payload[4:6]
@@ -1370,7 +1467,9 @@ def parser_31d9(payload, msg) -> Optional[dict]:
     bitmap = int(payload[2:4], 16)
 
     result = {
-        "exhaust_fan_speed": percent(payload[4:6]),  # NOTE: is 31DA/payload[38:40]
+        "exhaust_fan_speed": percent(
+            payload[4:6], high_res=True
+        ),  # NOTE: is 31DA/payload[38:40]
         "passive": bool(bitmap & 0x02),
         "damper_only": bool(bitmap & 0x04),
         "filter_dirty": bool(bitmap & 0x20),
@@ -1426,16 +1525,6 @@ def parser_31da(payload, msg) -> Optional[dict]:
         0x18: "auto",
     }
 
-    def percent(val, precision=0.5) -> Optional[float]:
-        if val in ("EF", "FF"):
-            return
-        if len(val) != 2:
-            raise TypeError(val)
-        result = int(val, 16)
-        if result > 100 / precision:
-            raise ValueError(result)
-        return result / 100 / precision
-
     # I --- 37:261128 --:------ 37:261128 31DA 029 00004007D045EF7FFF7FFF7FFF7FFFF808EF03C8000000EFEF7FFF7FFF
     # I --- 37:053679 --:------ 37:053679 31DA 030 00EF007FFF41EF7FFF7FFF7FFF7FFFF800EF0134000000EFEF7FFF7FFF00
 
@@ -1467,8 +1556,8 @@ def parser_31da(payload, msg) -> Optional[dict]:
         "air_quality": percent(payload[2:4]),
         "air_quality_base": int(payload[4:6], 16),  # NOTE: 12C8/payload[4:6]
         "co2_level": double(payload[6:10]),  # ppm NOTE: 1298/payload[2:6]
-        "indoor_humidity": percent(payload[10:12], precision=1),  # TODO: 12A0?
-        "outdoor_humidity": percent(payload[12:14], precision=1),
+        "indoor_humidity": percent(payload[10:12], high_res=False),  # TODO: 12A0?
+        "outdoor_humidity": percent(payload[12:14], high_res=False),
         "exhaust_temperature": double(payload[14:18], factor=100),
         "supply_temperature": double(payload[18:22], factor=100),
         "indoor_temperature": double(payload[22:26], factor=100),
@@ -1476,11 +1565,13 @@ def parser_31da(payload, msg) -> Optional[dict]:
         "speed_cap": int(payload[30:34], 16),
         "bypass_pos": percent(payload[34:36]),
         "fan_info": CODE_31DA_FAN_INFO[int(payload[36:38], 16) & 0x1F],
-        "exhaust_fan_speed": percent(payload[38:40]),  # NOTE: 31D9/payload[4:6]
-        "supply_fan_speed": percent(payload[40:42]),
+        "exhaust_fan_speed": percent(
+            payload[38:40], high_res=True
+        ),  # NOTE: 31D9/payload[4:6]
+        "supply_fan_speed": percent(payload[40:42], high_res=True),
         "remaining_time": double(payload[42:46]),  # mins NOTE: 22F3/payload[2:6]
-        "post_heat": percent(payload[46:48]),
-        "pre_heat": percent(payload[48:50]),
+        "post_heat": percent(payload[46:48], high_res=False),
+        "pre_heat": percent(payload[48:50], high_res=False),
         "supply_flow": double(payload[50:54], factor=100),  # L/sec
         "exhaust_flow": double(payload[54:58], factor=100),  # L/sec
     }
@@ -1488,8 +1579,8 @@ def parser_31da(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # ventilation heater?
 def parser_31e0(payload, msg) -> dict:
+    """Notes are.
 
-    """
     van means “of”.
     - 0 = min. van min. potm would be:
     - 0 = minimum of minimum potentiometer
@@ -1528,14 +1619,28 @@ def parser_31e0(payload, msg) -> dict:
         break;
     }
     """
+    #  I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
+    #  I --- 32:166025 --:------ 30:079129 31E0 004 00-00-00-00
+    #  I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
+    #  I --- 32:168090 30:082155 --:------ 31E0 004 00-00-C8-00
 
     return {
         "active": bool_from_hex(payload[4:6]),
         "_unknown_0": payload[:4],
-        "_unknown_2": payload[6:],
+        "unknown_2": payload[6:],
     }
 
 
+@parser_decorator  # supplied boiler water (flow) temp
+def parser_3200(payload, msg) -> Optional[dict]:
+    return {"temperature": temp_from_hex(payload[2:])}
+
+
+@parser_decorator  # return (boiler) water temp
+def parser_3210(payload, msg) -> Optional[dict]:
+    return {"temperature": temp_from_hex(payload[2:])}
+
+
 @parser_decorator  # opentherm_msg
 def parser_3220(payload, msg) -> Optional[dict]:
 
@@ -1587,6 +1692,13 @@ def parser_3220(payload, msg) -> Optional[dict]:
     return result
 
 
+# @parser_decorator  # R8810A/20A
+# def parser_3221(payload, msg) -> Optional[dict]:
+
+#     # 2021-11-03T09:55:43.112792 071 RP --- 10:052644 18:198151 --:------ 3221 002 000F
+#     # 2021-11-02T05:15:55.767108 046 RP --- 10:048122 18:006402 --:------ 3221 002 0000
+
+
 @parser_decorator  # actuator_sync (aka sync_tpi: TPI cycle sync)
 def parser_3b00(payload, msg) -> Optional[dict]:
     # system timing master: the device that sends I/FCC8 pkt controls the heater relay
@@ -1610,14 +1722,18 @@ def parser_3b00(payload, msg) -> Optional[dict]:
     # 064  I --- 01:078710 --:------ 01:078710 3B00 002 FCC8
 
     def complex_index(payload, msg) -> dict:  # has complex idx
-        if msg.verb == I_ and msg.src.type in ("01", "23") and msg.src is msg.dst:
+        if (
+            msg.verb == I_ and msg.src.type in ("01", "23") and msg.src is msg.dst
+        ):  # DEX
             assert payload[:2] == "FC"
             return {"domain_id": "FC"}
         assert payload[:2] == "00"
         return {}
 
     assert msg.len == 2, msg.len
-    assert payload[:2] == {"01": "FC", "13": "00", "23": "FC"}.get(msg.src.type, "00")
+    assert payload[:2] == {"01": "FC", "13": "00", "23": "FC"}.get(
+        msg.src.type, "00"
+    )  # DEX
     assert payload[2:] == "C8", payload[2:]  # Could it be a percentage?
 
     return {
@@ -1629,7 +1745,7 @@ def parser_3b00(payload, msg) -> Optional[dict]:
 @parser_decorator  # actuator_state
 def parser_3ef0(payload, msg) -> dict:
 
-    if msg.src.type in "08":  # Honeywell Jasper ?HVAC
+    if msg.src.type in "08":  # Honeywell Jasper ?HVAC, DEX
         assert msg.len == 20, f"expecting len 20, got: {msg.len}"
         return {
             "ordinal": f"0x{payload[2:8]}",
@@ -1644,15 +1760,16 @@ def parser_3ef0(payload, msg) -> dict:
         # .I --- 13:023770 --:------ 13:023770 3EF0 003 00C8FF
         assert payload[2:4] in ("00", "C8"), f"byte 1: {payload[2:4]}"
         assert payload[4:6] == "FF", f"byte 2: {payload[4:6]}"
+        mod_level = percent(payload[2:4])
 
     if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
         # RP --- 10:105624 01:133689 --:------ 3EF0 006 0000100000FF
         # RP --- 10:105624 01:133689 --:------ 3EF0 006 003B100C00FF
-        assert int(payload[2:4], 16) <= 100, f"byte 1: {payload[2:4]}"
         assert payload[4:6] in ("10", "11"), f"byte 2: {payload[4:6]}"
+        mod_level = percent(payload[2:4], high_res=False)
 
     result = {
-        "modulation_level": percent(payload[2:4]),  # TODO: rel_modulation_level
+        "modulation_level": mod_level,
         "_flags_0": payload[4:6],
     }
 
@@ -1660,24 +1777,8 @@ def parser_3ef0(payload, msg) -> dict:
         # RP --- 10:138822 01:187666 --:------ 3EF0 006 000110FA00FF  # ?corrupt
 
         # for OTB (there's no reliable) modulation_level <-> flame_state)
-        assert payload[6:8] in (
-            "00",
-            "01",
-            "02",
-            "04",
-            "08",
-            "0A",
-            "0C",
-            "42",
-        ), f"byte 3: {payload[6:8]}"
         assert int(payload[6:8], 16) & 0b11110000 == 0, f"byte 3: {payload[6:8]}"
-        assert payload[8:10] in (
-            "00",
-            "01",
-            "0A",
-            "FA",
-            "FF",
-        ), f"byte 4: {payload[8:10]}"
+        assert int(payload[8:10], 16) & 0b11110000 == 0, f"byte 4: {payload[8:10]}"
         assert payload[10:12] in ("00", "1C", "FF"), f"byte 5: {payload[10:12]}"
 
         result.update(
@@ -1687,14 +1788,14 @@ def parser_3ef0(payload, msg) -> dict:
                 "dhw_active": bool(int(payload[6:8], 0x10) & 1 << 2),
                 "flame_active": bool(int(payload[6:8], 0x10) & 1 << 3),
                 "_unknown_4": payload[8:10],
-                "_unknown_5": payload[10:12],
+                "_unknown_5": payload[10:12],  # rel_modulation?
             }
         )
 
-    if msg.len >= 9:  # I/RP|OTB|009
+    if msg.len >= 9:  # I/RP|OTB|009 (R8820A only?)
         assert int(payload[12:14], 16) & 0b11111100 == 0, f"byte 6: {payload[12:14]}"
         assert int(payload[12:14], 16) & 0b00000010 == 2, f"byte 6: {payload[12:14]}"
-        assert 10 <= int(payload[14:16], 16) <= 80, f"byte 7: {payload[14:16]}"
+        assert 10 <= int(payload[14:16], 16) <= 90, f"byte 7: {payload[14:16]}"
         assert int(payload[16:18], 16) in (0, 100), f"byte 8: {payload[18:]}"
 
         result.update(
@@ -1702,7 +1803,7 @@ def parser_3ef0(payload, msg) -> dict:
                 "_flags_6": flag8(payload[12:14]),
                 "ch_active": bool(int(payload[12:14], 0x10) & 1 << 0),
                 "ch_setpoint": int(payload[14:16], 0x10),
-                "max_rel_modulation": int(payload[16:18], 0x10),
+                "max_rel_modulation": percent(payload[16:18], high_res=False),
             }
         )
 
@@ -1712,14 +1813,14 @@ def parser_3ef0(payload, msg) -> dict:
 @parser_decorator  # actuator_cycle
 def parser_3ef1(payload, msg) -> dict:
 
-    if msg.src.type == "08":  # Honeywell Jasper ?HVAC
+    if msg.src.type == "08":  # Honeywell Jasper ?HVAC, DEX
         assert msg.len == 18, f"expecting len 18, got: {msg.len}"
         return {
             "ordinal": f"0x{payload[2:8]}",
             "blob": payload[8:],
         }
 
-    if msg.src.type == "31":  # and msg.len == 12:  # or (12, 20) Honeywell Japser ?HVAC
+    if msg.src.type == "31":  # and msg.len == 12:  # or (12, 20) Japser ?HVAC, DEX
         assert msg.len == 12, f"expecting len 12, got: {msg.len}"
         return {
             "ordinal": f"0x{payload[2:8]}",
@@ -1730,11 +1831,11 @@ def parser_3ef1(payload, msg) -> dict:
         # assert (
         #     re.compile(r"^00[0-9A-F]{10}FF").match(payload)
         # ), "doesn't match: " + r"^00[0-9A-F]{10}FF"
-        assert int(payload[2:6], 16) <= 7200, int(payload[2:6], 16)
-        assert payload[6:10] in ("87B3", "9DFA", "DCE1", "E638", "F8F7") or (
-            int(payload[6:10], 16) <= 7200
-        ), int(payload[6:10], 16)
-        assert percent(payload[10:12]) in (0, 1), percent(payload[10:12])
+        assert int(payload[2:6], 16) <= 7200, payload[2:6]
+        # assert payload[6:10] in ("87B3", "9DFA", "DCE1", "E638", "F8F7") or (
+        #     int(payload[6:10], 16) <= 7200
+        # ), payload[6:10]
+        assert percent(payload[10:12]) in (0, 1), payload[10:12]
 
     else:  # is OTB?
         # assert (
@@ -1756,38 +1857,55 @@ def parser_3ef1(payload, msg) -> dict:
 
 # @parser_decorator  # faked puzzle pkt shouldn't be decorated
 def parser_7fff(payload, msg) -> Optional[dict]:
-    LOOKUP = {"01": "ramses_rf", "02": "impersonating", "03": "message"}
 
-    if payload[2:4] == "00":
+    if payload[:2] != "00":
+        _LOGGER.debug("Invalid/deprecated Puzzle packet")
         return {
-            "datetime": dts_from_hex(payload[4:16]),
-            "message": str_from_hex(payload[18:]),
-            "_parser": f"v{VERSION}",
+            "msg_type": payload[:2],
+            "payload": str_from_hex(payload[2:]),
         }
 
-    elif payload[2:4] in LOOKUP:
+    if payload[2:4] not in LOOKUP_PUZZ:
+        _LOGGER.debug("Invalid/deprecated Puzzle packet")
         return {
-            LOOKUP[payload[2:4]]: str_from_hex(payload[4:]),
-            "_parser": f"v{VERSION}",
+            "msg_type": payload[2:4],
+            "message": str_from_hex(payload[4:]),
         }
 
-    elif payload[2:4] == "7F":
-        return {
-            "datetime": dts_from_hex(payload[4:16]),
-            "counter": int(payload[18:22], 16),
-            "interval": int(payload[24:28], 16) / 100,
-            "_parser": f"v{VERSION}",
-        }
-    return {
-        "header": payload[2:4],
-        "payload": payload[4:],
-        "_parser": f"v{VERSION}",
-    }
+    result = {}
+    if payload[2:4] != "13":
+        dtm = dt.fromtimestamp(int(payload[4:16], 16) / 1000)
+        result["datetime"] = dtm.isoformat(timespec="milliseconds")
+
+    msg_type = LOOKUP_PUZZ.get(payload[2:4], "message")
+
+    if payload[2:4] == "11":
+        msg = str_from_hex(payload[16:])
+        result[msg_type] = f"{msg[:4]}|{msg[4:6]}|{msg[6:]}"
+
+    elif payload[2:4] == "13":
+        result[msg_type] = str_from_hex(payload[4:])
+
+    else:
+        result[msg_type] = str_from_hex(payload[16:])
+
+    return {**result, "parser": f"v{VERSION}"}
 
 
 @parser_decorator
 def parser_unknown(payload, msg) -> Optional[dict]:
     # TODO: it may be useful to generically search payloads for hex_ids, commands, etc.
+
+    if msg.len == 2 and payload[:2] == "00":
+        return {
+            "_value": {"00": False, "C8": True}.get(payload[2:], int(payload[2:], 16))
+        }
+
+    if msg.len == 3 and payload[:2] == "00":
+        return {
+            "_value": temp_from_hex(payload[2:]),
+        }
+
     raise NotImplementedError
 
 
